import base64
from Crypto.Cipher import AES

from pyaes256.common import SALT_PREFIX, SALT_LENGTH_BYTES, pbkdf2_derive_key_iv


def decrypt(encrypted_base64_text, password, show_key=False):
    prefix_bytes = len(SALT_PREFIX) + SALT_LENGTH_BYTES
    decoded_base64_msg = base64.b64decode(encrypted_base64_text)
    salt = decoded_base64_msg[8:16]  # parse salt
    encrypted_text = decoded_base64_msg[prefix_bytes:]
    aes_key, iv = pbkdf2_derive_key_iv(password, salt, show_key=show_key)
    cypher = AES.new(aes_key, AES.MODE_CBC, iv)
    decrypted_with_pad = cypher.decrypt(encrypted_text)
    decrypted = unpad(decrypted_with_pad).decode('utf-8')
    return decrypted


def unpad(byte_string):
    """
    Remove the PKCS#7 padding from a text bytestring.
    """
    val = byte_string[-1]
    if val > AES.block_size:
        raise ValueError('Input is not padded or padding is corrupt')
    return byte_string[:(len(byte_string) - val)]



from Crypto.Cipher import AES
import secrets
import base64

from pyaes256.common import SALT_LENGTH_BYTES, SALT_PREFIX, pbkdf2_derive_key_iv


def encrypt(text, password, salt=None, show_key=False):
    """
    Encrypts a text the same way as running openssl enc
    :param show_key: log derived key and iv if set to true
    :param text: your text you want to encrypt
    :param password: your password to encrypt with. We are using pkdfd2 to derive the aes key from the password
    :param salt:
    :return: base64 encoded encrypted text
    """
    if salt is None:
        salt = secrets.token_bytes(SALT_LENGTH_BYTES)
    else:
        salt = bytearray.fromhex(salt)

    aes_key, iv = pbkdf2_derive_key_iv(password, salt, show_key=show_key)
    print(f'\nsalt={salt.hex()}')
    cypher = AES.new(aes_key, AES.MODE_CBC, iv)
    padded = pad(text.encode())
    msg = cypher.encrypt(padded)
    base64_msg = base64.b64encode(SALT_PREFIX + salt + msg)
    return base64_msg.decode('utf-8')


def pad(byte_string):
    """
    Pad an input bytestring according to PKCS#7
    :param byte_string:
    :return: padded string
    """
    val = AES.block_size - (len(byte_string) % AES.block_size)
    return byte_string + bytearray([val] * val)
